@@ -15,7 +15,7 @@ from gwalk.utils.multivariate_normal import std_of_params
 from gwalk.utils.multivariate_normal import cor_of_params
 from gwalk.utils.multivariate_normal import cov_of_params
 from gwalk.utils.multivariate_normal import mu_cov_of_params
-from gwalk.utils.multivariate_normal import params_of_mu_cov
+from gwalk.utils.multivariate_normal import params_of_norm_mu_cov
 from gwalk.utils.multivariate_normal import multivariate_normal_pdf
 from gwalk.utils.multivariate_normal import n_param_of_ndim
 from gwalk.utils.multivariate_normal import ndim_of_n_param
@@ -27,11 +27,13 @@ from gwalk.utils.multivariate_normal import multivariate_normal_marginal2d
 from relative_entropy_cython import relative_entropy
 
 #### Public Imports ####
+import warnings
 import numpy as np
 #### Globals ####
 _INV_RT2 = 1./np.sqrt(2)
 _LOG_2PI = np.log(2*np.pi)
 DEFAULT_SIG_MAX = 3.
+DEFAULT_NORM_MAX = 10.
 
 ######## Multivariable Normal Object ########
 class MultivariateNormal(Relation):
@@ -45,6 +47,7 @@ class MultivariateNormal(Relation):
                  scale,
                  seed=None,
                  sig_max = None,
+                 norm = None,
                 ):
         '''Initialize a Multivariate Normal Relation
 
@@ -63,6 +66,10 @@ class MultivariateNormal(Relation):
             Input seed for random number generator
         sig_max: float, optional
             Input sets limit on variance parameters
+        fit_norm: boolean, optional
+            Input decide if to fit a normalization parameter
+        max_norm: float, optional
+            Input decide the maximum value for the normalization parameter
         '''
         # Check random state
         self.rs = np.random.RandomState(seed)
@@ -88,8 +95,19 @@ class MultivariateNormal(Relation):
         # Generate a temporary relationship with the variables
         temp = Relation(variables,random_state=self.rs)
 
-        # Inialize a Relation object with the mu variables
+        # Initialize this object as a "Relation"
         super().__init__(random_state=self.rs)
+
+        # Create a normalization relation
+        norm_parameters = Relation(random_state=self.rs)
+        if norm is None:
+            p = Parameter("norm", 1.0, [0., DEFAULT_NORM_MAX], label="norm")
+            norm_parameters.add_parameter(p)
+        else:
+            norm_parameters.add_parameter(norm)
+
+
+        # Inialize a Relation object with the mu variables
         mu_parameters = Relation(random_state=self.rs)
 
         # Rename mu variables
@@ -103,7 +121,7 @@ class MultivariateNormal(Relation):
             mu_parameters.add_parameter(p)
 
         # Contain the entire list of covariance parameters
-        temp_obj = mu_parameters + cov_parameters
+        temp_obj = norm_parameters + mu_parameters + cov_parameters
         self.__dict__.update(temp_obj.__dict__)
 
         # Carry functions that identify things about the covariance matrix
@@ -117,6 +135,9 @@ class MultivariateNormal(Relation):
 
     #### Tools for working with this object ####
     ## Guess reading tools ##
+    def read_norm(self):
+        ''' Read the normalization constant'''
+        return self._parameters["norm"].guess
 
     def read_mu(self):
         ''' Read the mu guesses'''
@@ -197,7 +218,7 @@ class MultivariateNormal(Relation):
         '''
         return mu_cov_of_params(X)
 
-    def params_of_mu_cov(mu,cov):
+    def params_of_norm_mu_cov(mu,cov):
         '''Convert parameter values to mu and cov values
 
         Parameters
@@ -207,7 +228,7 @@ class MultivariateNormal(Relation):
         cov: array like, shape = (npts, ndim, ndim)
             Input Array of cov values
         '''
-        return params_of_mu_cov(mu,cov)
+        return params_of_norm_mu_cov(mu,cov)
 
     ## Guess assignment ##
 
@@ -226,10 +247,12 @@ class MultivariateNormal(Relation):
         # Fix data
         Xs = self.check_sample(Xs)
         # Check mu parameters
-        c1 = self.summands[0].satisfies_constraints(Xs[:,:self.ndim])
+        c0 = self.summands[0].satisfies_constraints(Xs[:,0])
+        # Check mu parameters
+        c1 = self.summands[1].satisfies_constraints(Xs[:,1:self.ndim+1])
         # Check covariance
-        c2 = self.summands[1].satisfies_constraints(Xs[:,self.ndim:])
-        constrained_indx = (c1 & c2)
+        c2 = self.summands[2].satisfies_constraints(Xs[:,self.ndim+1:])
+        constrained_indx = (c0 & c1 & c2)
         # Print statement for errors
         #if (constrained_indx.size == 1) and (not constrained_indx):
         #    print(c1, c2, c3)
@@ -246,6 +269,7 @@ class MultivariateNormal(Relation):
                    scale=False,
                    indices=None,
                    limits=None,
+                   lnL_offset=None,
                   ):
         '''Find the likelihood of some data with a particular guess of parameters
         
@@ -330,29 +354,31 @@ class MultivariateNormal(Relation):
 
         # Consider bounds
         if limits is None:
-            limits = self.summands[0].read_limits()*self.scale[:,None]
+            limits = self.summands[1].read_limits()*self.scale[:,None]
         # If only one set of limits exists, this is easy
         if len(limits.shape) == 2:
             # Check for sensible limits
             assert (limits.shape[0] == self.ndim) and (limits.shape[1] == 2)
+            limits = limits[indices,:]
             # generate the keep index
             Y_keep = np.ones((Y.shape[0]),dtype=bool)
-            for i in range(self.ndim):
+            for i in range(len(indices)):
                 Y_keep &= Y[:,i] >= limits[i,0]
                 Y_keep &= Y[:,i] <= limits[i,1]
-            L[:,Y_keep] = 0.
+            L[:,~Y_keep] = 0.
 
         elif len(limits.shape) == 3:
             # Check for sensible limits
             assert limits.shape[0] == n_gauss
             assert limits.shape[1] == self.ndim
             assert limits.shape[2] == 2
+            limits = limits[:,indices,:]
             Y_keep = np.ones_like(L,dtype=bool)
             for i in range(n_gauss):
-                for j in range(self.ndim):
+                for j in range(len(indices)):
                     Y_keep[i] &= Y[:,j] >= limits[i,j,0]
                     Y_keep[i] &= Y[:,j] <= limits[i,j,1]
-            L[Y_keep] = 0.
+            L[~Y_keep] = 0.
 
         # Apply weights
         if not (w is None):
@@ -368,6 +394,8 @@ class MultivariateNormal(Relation):
                 L = np.sum(L, axis=1)
             else:
                 L = np.prod(L, axis=1)
+        elif (log_scale and lnL_offset):
+            L += lnL_offset
 
         return L
 
@@ -474,7 +502,7 @@ class MultivariateNormal(Relation):
             # Draw that many samples
             Xs = self.sample_normal_unconstrained(rv, n_need)
             # Check if these samples satisfy our constraints
-            k_indx = self.summands[0].satisfies_constraints(Xs)
+            k_indx = self.summands[1].satisfies_constraints(Xs)
             # Identify the number of new samples to keep from this iteration
             n_it = np.sum(k_indx)
             # Keep iterating if there are not valid samples
@@ -511,12 +539,13 @@ class MultivariateNormal(Relation):
         import numpy as np
 
         # Find the average and covariance of the samples
+        norm=self.read_guess()[0]
         Y = Y/self.scale
         mean = np.average(Y, weights = w, axis = 0)
         cov = np.cov(Y.T, aweights = w)
 
         # convert to a sample
-        X = params_of_mu_cov(mean, cov)
+        X = params_of_norm_mu_cov(norm, mean, cov)
 
         # Assign this guess
         if assign:
@@ -541,11 +570,13 @@ class MultivariateNormal(Relation):
         # Open Database pointer
         db = Database(fname, group=label)
         # Calculate things
+        norm = self.read_norm()
         mu, cov = self.read_physical()
         cor = self.read_cor()
         std = self.read_std()*self.scale
-        limits = self.summands[0].read_limits()*self.scale[:,None]
+        limits = self.summands[1].read_limits()*self.scale[:,None]
         # Assign things
+        db.dset_set("norm",norm)
         db.dset_set("mean",mu)
         db.dset_set("std",std)
         db.dset_set("cor",cor)
@@ -555,15 +586,16 @@ class MultivariateNormal(Relation):
         # Assign attrs
         db.attr_set(".","seed",self.seed)
         db.attr_set(".","sig_max",self.sig_max)
+        db.attr_set("norm","max",self._parameters["norm"].limits[1])
         if not (attrs is None):
             for item in attrs:
                 db.attr_set(".",item,attrs[item])
         # Get names
         names = []
         labels = []
-        for i, key in enumerate(self.summands[0]._parameters):
-            names.append(self.summands[0]._parameters[key].name)
-            labels.append(self.summands[0]._parameters[key].label)
+        for i, key in enumerate(self.summands[1]._parameters):
+            names.append(self.summands[1]._parameters[key].name)
+            labels.append(self.summands[1]._parameters[key].label)
         db.attr_set(".", "names", names)
         db.attr_set(".", "labels", labels)
 
@@ -606,7 +638,17 @@ class MultivariateNormal(Relation):
         db = Database(fname)
         assert db.exists(label)
         db = Database(fname, group=label)
-
+        
+        # load norm param
+        try:
+            norm = db.dset_value("norm")
+            # Load norm max
+            norm_max = db.attr_value("norm","max")
+        except:
+            warnings.warn(
+                "Warning: no normalization constant found. Assuming 1.0")
+            norm = 1.0
+            norm_max=DEFAULT_NORM_MAX
         # Load mu params
         mu = db.dset_value("mean")
         # load sig params
@@ -653,18 +695,22 @@ class MultivariateNormal(Relation):
         else:
             sig_max = DEFAULT_SIG_MAX
 
+        # Create norm parameter
+        pnorm = Parameter("norm", norm, [0.,norm_max],"norm")
+
         # Initialize object
         MV = MultivariateNormal(
                                 variables,
                                 scale,
                                 seed=seed,
-                                sig_max=sig_max
+                                sig_max=sig_max,
+                                norm=pnorm,
                                )
 
         # Set params
         _mu_scaled = mu/scale
         _Sig_scaled = cov/np.outer(scale,scale)
-        params = params_of_mu_cov(_mu_scaled, _Sig_scaled).flatten()
+        params = params_of_norm_mu_cov(norm, _mu_scaled, _Sig_scaled).flatten()
 
         # Assign params
         MV.assign_guess(params)
