@@ -235,7 +235,7 @@ class Database(object):
                             Head.create_group(self.group)
 
                         # Copy group attributes
-                        group_attrs = self.attr_list('/')
+                        group_attrs = self.attr_list('.')
                         for item in group_attrs:
                             Head[self.group].attrs[item] = Origin[self.group].attrs[item]
 
