@@ -98,7 +98,7 @@ def n_param_of_ndim(ndim):
     ndim: int
         Input number of dimensions
     '''
-    return int((ndim*ndim + 3*ndim)//2)
+    return int((ndim*ndim + 3*ndim)//2) + 1
 
 def ndim_of_n_param(n_param, max_dim=100):
     '''find the dimensionality of a guess
@@ -132,7 +132,7 @@ def params_reduce_1d(X, index):
     # Protect against invalid choices
     assert ndim > index
     # Reduce dimensionality
-    Xi = np.asarray([X[:,index],X[:,index+ndim]]).T
+    Xi = np.asarray([X[:,0], X[:,index + 1],X[:,index+ndim + 1]]).T
     return Xi
 
 def params_reduce_2d(X, index, jndex):
@@ -157,11 +157,12 @@ def params_reduce_2d(X, index, jndex):
     assert index > jndex
     # Reduce dimensionality
     Xij = np.asarray([
-                      X[:,index],
-                      X[:,jndex],
-                      X[:,index+ndim],
-                      X[:,jndex+ndim],
-                      X[:,2*ndim + ((index*(index-1))//2) + jndex]
+                      X[:,0],
+                      X[:,index + 1],
+                      X[:,jndex + 1],
+                      X[:,index+ndim + 1],
+                      X[:,jndex+ndim + 1],
+                      X[:,2*ndim + ((index*(index-1))//2) + jndex + 1]
                      ]).T
     return Xij
 
@@ -185,10 +186,12 @@ def params_reduce_dd(X, indices):
     n_param_index = n_param_of_ndim(ndim_index)
     # Initialize array
     Xp = np.empty((n_gauss, n_param_index))
+    # Set normalization
+    Xp[:,0] = X[:,0]
     # Loop through Xp dimensions
     for i in range(ndim_index):
-         Xp[:,i] =              X[:,indices[i]]
-         Xp[:,i+ndim_index] =   X[:,indices[i]+ndim]
+         Xp[:,i+1] =              X[:,indices[i] + 1]
+         Xp[:,i+ndim_index+1] =   X[:,indices[i]+ndim + 1]
     # Loop through correlation factors
     for i in range(ndim_index):
         for j in range(i):
@@ -198,8 +201,8 @@ def params_reduce_dd(X, indices):
                 index, jndex = indices[j], indices[i]
             else:
                 raise RuntimeError("indices[i] == indices[j]")
-            Xp[:,2*ndim_index + ((i*(i-1))//2) + j] = \
-                X[:,2*ndim + ((index*(index-1))//2) + jndex]
+            Xp[:,2*ndim_index + ((i*(i-1))//2) + j + 1] = \
+                X[:,2*ndim + ((index*(index-1))//2) + jndex + 1]
     return Xp
 
                 
@@ -208,7 +211,7 @@ def params_reduce_dd(X, indices):
 
 ## Conversions ##
 
-def mu_of_params(X):
+def mu_of_params(X,scale=None,**kwargs):
     '''Reconstruct the vectorized mu parameters of input Gaussian parameters
     Parameters
     ----------
@@ -218,16 +221,21 @@ def mu_of_params(X):
     # Protect against single set of parameters
     if len(X.shape) == 1:
         X = X[None,:]
+    if not (scale is None):
+        if len(scale.shape) == 1:
+            scale = scale[None,:]
     # Identify n_gauss
     n_gauss = X.shape[0]
     # Identify ndim
     n_param = X.shape[1]
     ndim = ndim_of_n_param(n_param)
     # Reconstruct mu
-    mu = X[:,:ndim]
+    mu = X[:,1:ndim+1]
+    if not (scale is None):
+        mu*=scale
     return mu
 
-def std_of_params(X):
+def std_of_params(X,scale=None,**kwargs):
     '''Reconstruct the vectorized sigma parameters of input Gaussian parameters
     Parameters
     ----------
@@ -237,16 +245,21 @@ def std_of_params(X):
     # Protect against single set of parameters
     if len(X.shape) == 1:
         X = X[None,:]
+    if not (scale is None):
+        if len(scale.shape) == 1:
+            scale = scale[None,:]
     # Identify n_gauss
     n_gauss = X.shape[0]
     # Identify ndim
     n_param = X.shape[1]
     ndim = ndim_of_n_param(n_param)
     # Reconstruct std
-    std = X[:,ndim:2*ndim]
+    std = X[:,ndim+1:2*ndim+1]
+    if not (scale is None):
+        std*=scale
     return std
 
-def cor_of_params(X):
+def cor_of_params(X,**kwargs):
     '''Reconstruct the vectorized correlation matrix of input Gaussian parameters
     Parameters
     ----------
@@ -267,7 +280,7 @@ def cor_of_params(X):
     cor = np.ones((n_gauss, ndim, ndim))*np.eye(ndim)
     # The correlation index is carefully kept consistent
     # DO NOT CHANGE THIS!
-    cor_index = 2*ndim
+    cor_index = 2*ndim + 1
     for i in range(ndim):
         for j in range(i):
             cor[:,i,j] = cor[:,j,i] = X[:,cor_index]
@@ -276,7 +289,7 @@ def cor_of_params(X):
         raise RuntimeError("Correlation Matrix reconstruction is broken")
     return cor
 
-def cov_of_params(X):
+def cov_of_params(X,**kwargs):
     '''Reconstruct the vectorized corvariance of input Gaussian parameters
     Parameters
     ----------
@@ -292,9 +305,9 @@ def cov_of_params(X):
     n_param = X.shape[1]
     ndim = ndim_of_n_param(n_param)
     # Calculate std of params
-    std = std_of_params(X)
+    std = std_of_params(X,**kwargs)
     # Calculate correlation matrix
-    cor = cor_of_params(X)
+    cor = cor_of_params(X,**kwargs)
     # Reconstruct the variance matrix
     # DO NOT CHANGE THIS!
     std_expand = np.tensordot(std, np.ones(ndim), axes=0)
@@ -303,18 +316,18 @@ def cov_of_params(X):
     cov = cor * var
     return cov
 
-def mu_cov_of_params(X):
+def mu_cov_of_params(X,**kwargs):
     '''Reconstruct vectorized mu and covariance of input Gaussian parameters
     Parameters
     ----------
     X: array like, shape = (npts, nparams)
         Input Array of parameter guesses
     '''
-    mu = mu_of_params(X)
-    cov = cov_of_params(X)
+    mu = mu_of_params(X,**kwargs)
+    cov = cov_of_params(X,**kwargs)
     return mu, cov
 
-def params_of_mu_cov(mu, cov):
+def params_of_norm_mu_cov(norm, mu, cov):
     '''Reconstruct Gaussian vectorized parameters for given mu and covariance
     Parameters
     ----------
@@ -338,14 +351,16 @@ def params_of_mu_cov(mu, cov):
         raise ValueError("cov is not shape (n_gauss, ndim, ndim)")
     # Initialize array
     X = np.empty((n_gauss,n_param))
+    # Initialize normalization
+    X[:,0] = norm
     # Insert mu values
-    X[:,:ndim] = mu
+    X[:,1:ndim+1] = mu
     # Insert std values
-    X[:,ndim:2*ndim] = np.sqrt(np.sum(cov*np.eye(ndim),axis=1))
+    X[:,ndim+1:2*ndim+1] = np.sqrt(np.sum(cov*np.eye(ndim),axis=1))
     ## Compute Correlation ##
     cor = cor_of_cov(cov)
     ## Insert Correlation Values ##
-    cor_index = 2*ndim
+    cor_index = 2*ndim +1
     for i in range(ndim):
         for j in range(i):
             X[:,cor_index] = cor[:,i,j]
@@ -471,11 +486,10 @@ def multivariate_normal_pdf(
         raise ValueError("Usage: scale.shape = %s"%str(scale.shape))
         
     ## Reconstruct mu and covariance ##
-    X_mu, X_cov = mu_cov_of_params(X)
+    X_mu, X_cov = mu_cov_of_params(X,scale=scale)
     X_mu = np.asarray(X_mu,order='c')
     #XA = params_of_mu_cov(X_mu, X_cov)
     #print(np.max(np.abs(X - XA)))
-    #raise Exception("Break")
 
     ## Eigenvector Decomposition ##
     # Find eigenvalues and eigenvectors (see scipy)
@@ -502,7 +516,7 @@ def multivariate_normal_pdf(
     U = u*s_pinv[:,None,:]
 
     # Calculate the mahalanobis factor
-    maha = mahalanobis_cython(np.asarray(Y,order='c'), scale, X_mu, U, n_gauss, n_pts, ndim)
+    maha = mahalanobis_cython(np.asarray(Y,order='c'), X_mu, U, n_gauss, n_pts, ndim)
     # Compute the likelihood!
     L = pdf_exp_product(maha, log_det_cov, ndim*_LOG_2PI, n_gauss, n_pts, ndim, log_scale=log_scale)
     L[~keep] = 0.
